
% ***********************************************************************
% Set SpeDeBox to Matlab's search path
% ***********************************************************************

% Before performing the analysis, make sure that the toolbox
% and all its subfolders are set to Matlab's path. It is also highly recommended
% to set the directory of the toolbox as the Matlab's current directory.
% If these steps have not been already carried out, below is a script
% to perform this. Verify that the path is correct in case you
% have several copies of the toolbox available in your file system.

z = which('analysis_example1.m'); % Find full filename to this m-file
f = fileparts(z); % separate filename and pathname
disp('Verify that the toolbox is in the following path:')
disp(f) % visualize pathname
addpath(genpath(f)) % add path and all its subfolders to Matlab's path
cd(f) % set toolbox directory as current Matlab's directory


% ***********************************************************************
% Generate synthetic data
% ***********************************************************************

T = 50; % total number of time-points in each epoch
C = 64; % total number of sensors
N = 500; % total number of epochs
K = 2; % total number of categories

% Generate category labels:
labels = 1 + floor(K*rand(1,N));

% Generate random data in Epoch format:
data = rand(C,N,T);

% add class-specific information to randomly selected channels:
for m = 1:K
    rnCol = randperm(size(data,1));
    data(rnCol(1:4),labels == m,:) = 0.1*m + data(rnCol(1:4),labels == m,:);
end


% ***********************************************************************
% Set preprocessing parameters
% ***********************************************************************

% Select preprocessing options:

% Set parameters of the STFT:
params.general.samplfreq = 1000; % sampling frequency in Hz
params.general.minfreq = 1; % minimum frequency of interest in Hz
params.general.maxfreq = 500; % maximum frequency of interest in Hz

params.general.do_ica = 0; % do not run Fourier-ICA


% ***********************************************************************
% Set classifier parameters
% ***********************************************************************

% set regularization sequences for the Sparse bilinear classifier:
params.sparse_bilinear.lambdas_comp = [10 20 30]; % regularization sequence of spatial coefficients
params.sparse_bilinear.lambdas_freq = [0]; % regularization sequence of frequency coefficients
% Note: Forcing higher values for lambdas_comp lead to more easily interpretable models
% with less number of relevant components.

% set rank (number of coefficient vectors) of the classifiers for the Sparse bilinear
% and the Three-way DA classifiers:
params.sparse_bilinear.modelRank = 1;
params.three_way_DA.modelRank = 3;


% ***********************************************************************
% Split data into training, validation and test data sets
% ***********************************************************************

% Split data set such that:
% the first 1/3 of the data is used for classifier training
% the second 1/3 of the data is used for validation of the classifier hyperparameters
% the last 1/3 of the data is used for evaluating the classifier performance for unseen data
% Note: This procedure for classifier testing is not comprehensive and could be improved
% e.g. by using a 10-fold cross-validation.

% number of samples used for training, validation and testing:
nr_samples_train = round(N/3);
nr_samples_validation = round(N/3);
nr_samples_test = round(N/3);

% train, validation and test data set indices:
samples_train = 1:nr_samples_train;
samples_validation = (nr_samples_train+1):(nr_samples_train+nr_samples_validation);
samples_test = (nr_samples_train+nr_samples_validation+1):N;

% obtain train, validation and test data sets:
data_train = data(:,samples_train,:); % training data set (first 1/3 of the data)
labels_train = labels(samples_train); % training labels

data_validation = data(:,samples_validation,:); % validation data set (second 1/3 of the data)
labels_validation = labels(samples_validation); % validation labels

data_test = data(:,samples_test,:); % test data set (last 1/3 of the data)
labels_test = labels(samples_test); % test labels


% ***********************************************************************
% Preprocess data using STFT
% ***********************************************************************

% The following function performs the selected preprocessing options (STFT, ICA).
% Outputs are Fourier-transformed complex-valued epochs, class labels of the epoch,
% and the estimated complex-valued mixing matrix (A).
[data_train,data_validation,data_test,labels_train,labels_validation,...
    labels_test,A] = preprocess_data(data_train,data_validation,...
    data_test,labels_train,labels_validation,labels_test,params);

% Take the absolute value of the Fourier coefficients as the classifier inputs:
data_train = abs(data_train);
data_validation = abs(data_validation);
data_test = abs(data_test);


% ***********************************************************************
% Classifier training and testing
% ***********************************************************************

% Train and validate Sparse bilinear classifier:
[Classifier_final1,resu1] = train_and_test_classifier(data_train,data_validation,data_test,labels_train,labels_validation,labels_test,'sparse_bilinear',params);
% Train and validate Spectral LDA classifier:
[Classifier_final2,resu2] = train_and_test_classifier(data_train,data_validation,data_test,labels_train,labels_validation,labels_test,'spectral_LDA',params);
% Train and validate Three-way DA classifier:
[Classifier_final3,resu3] = train_and_test_classifier(data_train,data_validation,data_test,labels_train,labels_validation,labels_test,'three_way_DA',params);


% ***********************************************************************
% Display classification accuracies
% ***********************************************************************

% Classification performance is saved in the structs:
% resu1		Sparse bilinear
% resu2		Spectral LDA
% resu3		Three-way DA
% Plot classification accuracy of the test data with the accuracy of two decimals:

disp(' ')
disp('Classification accuracy for the test data set:')
disp(['Sparse bilinear: ' num2str(round(resu1.acc*1e4)/1e2) '%'])
disp(['Spectral LDA   : ' num2str(round(resu2.acc*1e4)/1e2) '%'])
disp(['Three-way DA   : ' num2str(round(resu3.acc*1e4)/1e2) '%'])
disp(' ')

% ***********************************************************************