function [Vfinal,Wfinal,pairs] = bilinear_alternate(S,y,Ra,lambdas1,lambdas2)
% Training sparse bilinear classifier based on an alternating scheme of 
% linear logistic regression problems.
%
% Inputs: 
%
% S - input data of size F*N*C, 
% where F = number of frequency bins, N = number of epoch, C = number of channels/components
% y - label vector of size N containing ones (class one) and twos (class two).
% Ra - rank of the classifier
% lambdas1 - regularization parameter sequence 1
% lambdas2 - regularization parameter sequence 2
%
% Outputs:
% Vfinal - estimated "V" parameters 
% Wfinal - estimated "W" parameters
% pairs - regularization parameter pairs

% Set internal classifier parameters:
Iters = 10; % maximum number of alternations
Tol = 0.1; % tolerance to stop alternation
verbose = false; % set this flag to display information
L2on = true; % set this flag to use L2 instead of L1 (default) regularizer

N = length(y);
K = max(y);

% code class labels:
y2 = zeros(N, K);
for ii = 1:N
    y2(ii, y(ii)) = 1;
end
y2 = y2'; % K x N

% initialize model parameters to random values:
ma = max(S(:));
mi = min(S(:));
d1 = size(S,1);
d2 = size(S,3);
beta1 = (ma-mi)*rand(Ra*d1+1,1)+mi;
beta2 = (ma-mi)*rand(Ra*d2+1,1)+mi;

for k = 1:length(lambdas1)
    V(:,k) = beta1(:);
end

flag = 0;

lambda1 = ones(size(beta1));
lambda1(1,:) = 0;
lambda1 = lambda1(:);
lambda2 = ones(size(beta2));
lambda2(1,:) = 0;
lambda2 = lambda2(:);

% pairs of the regularization parameters:
nrPairs = length(lambdas1)*length(lambdas2);
pairs = zeros(nrPairs,2);
iter = 1;
for m = 1:length(lambdas1)
    for n = 1:length(lambdas2)
        pairs(iter,1) = m;
        pairs(iter,2) = n;
        iter = iter + 1;
    end
end
clear Wfinal Vfinal
Wfinal = zeros(length(beta2),nrPairs);
Vfinal = zeros(length(beta1),nrPairs);
allZeros = 0;

for m = 1:nrPairs % for each hyperparameter pair, train classifier    
    % obtain hyperparameters:
    lamV = lambdas1(pairs(m,1));
    lamW = lambdas2(pairs(m,2));
    V = beta1;
    W = beta2;
    
    % normalize coefficient vectors:
    V(2:end) = normalizeBilinearParams(V(2:end),Ra);
    W(2:end) = normalizeBilinearParams(W(2:end),Ra);
    
    clear nw nv Wall Vall
    ite = 1;
    for ite = 1:Iters % max number of alternations
        % linearize bilinear classification problem by multiplying from the left:
        [dataLinear,nonZeroCols,dimLinearData] = linearizeData(S,Ra,V(2:end,:),'left');
        [dataLinear] = addBiasTerms(dataLinear,nonZeroCols);
        nonZeroCols = [1 nonZeroCols + 1];        

        y = y(:);
        % optimize linear logistic regression model with L1 regularization:
        funObj = @(w)SoftmaxLoss2(w,dataLinear,y,K);
        options.verbose = 0;
        Wnew = L1General2_PSSgb(funObj,W(nonZeroCols),lamW*lambda2(nonZeroCols),options);
        
        % remove entire columns of the parameter matrix which are forced 
        % to zero (when rank > 1):
        WnewTmp = zeros(size(beta2));
        WnewTmp(nonZeroCols) = Wnew;
        Wnew = WnewTmp;
                
        Wnew(2:end) = normalizeBilinearParams(Wnew(2:end),Ra);        
        dif = Wnew(2:end,:) - W(2:end,:);
        nw(ite) = norm(dif(:));
        W = Wnew;
        % linearize bilinear classification problem by multiplying from the right:
        [dataLinear,nonZeroCols,dimLinearData] = linearizeData(S,Ra,W(2:end,:),'right');
        % add bias term:
        [dataLinear] = addBiasTerms(dataLinear,nonZeroCols);
        nonZeroCols = [1 nonZeroCols + 1];

        % optimize linear logistic regression model:
        funObj = @(w)SoftmaxLoss2(w,dataLinear,y,K);
        % There are two options for regularization, L2 and L1:
        if L2on 
            funObjL2 = @(w)penalizedL2(w,funObj,lamV*lambda1(nonZeroCols));
            mfOptions.Method = 'lfbgs';
            mfOptions.Display = 'off';
            Vnew = minFunc(funObjL2,V(nonZeroCols),mfOptions);
        else
            Vnew = L1General2_PSSgb(funObj,V(nonZeroCols),lamV*lambda1(nonZeroCols),options);
        end
               
        
        % remove entire columns of the parameter matrix which are forced
        % to zero (when rank > 1):
        VnewTmp = zeros(size(beta1));
        VnewTmp(nonZeroCols) = Vnew;
        Vnew = VnewTmp;
        
        Vnew(2:end) = normalizeBilinearParams(Vnew(2:end),Ra);

        % check difference between two consecutive parameter estimates:
        dif = Vnew(2:end,:) - V(2:end,:);
        nv(ite) = norm(dif(:));
        
        V = Vnew;        

        % save history of estimated classification coefficients:
        Vall(:,ite) = V;
        Wall(:,ite) = W;        
        
        if (length(W)-1)-sum(W==0) == 0
            allZeros = 1;
            break
        end
        
        % check convergence criterion:
        if nv(ite) < Tol && nw(ite) < Tol
            if verbose
                disp(['Alternation converged after ' num2str(ite) ' iterations.'])
            end
            break
        end
        ite = ite + 1;
    end
    
    [acc,labelsEstim,prob] = evaluate_prob_bilinear(S,y,W(2:end),V(2:end),Ra);
    if verbose
        disp(['Hyperparameter combination (' num2str(lamW) ',' num2str(lamV) ') accuracy: ' num2str(acc)])
    end
    ACC(m) = acc;
    if allZeros
        if verbose
            disp(['Hyperpyperparameters (' num2str(lamW) ',' num2str(lamV) ') forced all weights to zero!'])
        end
    end
    
    % save the last estimate for each hyperparameter combination:
    Wfinal(:,m) = W;
    Vfinal(:,m) = V;            
end


function W = normalizeBilinearParams(W,Ra)
% This function normalizes bilinear classifier's coefficient vectors.

if sum(abs(W))==0 || isnan(sum(abs(W)))
   W(1) = 1;
end

dimW = size(W,1)/Ra;
W = reshape(W,dimW,Ra);

for vv = 1:Ra
   if sum(abs(W(:,vv))~=0)
       W(:,vv) = W(:,vv)/norm(W(:,vv));
   end
end

W = W(:);

function [dataLinear,nonZeroCols] = addBiasTerms(dataLinear,nonZeroCols)

dataLinearTmp = ones(size(dataLinear,1),size(dataLinear,2)+1,size(dataLinear,3));
dataLinearTmp(:,2:end,:) = dataLinear;
nonZeroColsTmp = ones(size(nonZeroCols,1)+1,size(nonZeroCols,2));

dataLinearTmp = zeros(size(dataLinear,1),size(dataLinear,2)+1,size(dataLinear,3));
dataLinearTmp(:,2:end,:) = dataLinear;
nonZeroColsTmp = zeros(size(nonZeroCols,1)+1,size(nonZeroCols,2));

nonZeroColsTmp(2:end,:) = nonZeroCols;
nonZeroCols = nonZeroColsTmp;
dataLinear = dataLinearTmp;

function [dataLinear,nonZeroCols,dimLinearData] = linearizeData(data,Ra,M,direction)
% This function linearizes bilinear classification problem.

dimM = size(M,1)/Ra;
M = reshape(M,dimM,Ra);

if strcmp(direction,'right')
    dimData = size(data,1);
elseif strcmp(direction,'left')
    dimData = size(data,3);
else
    error('direction must be "left" or "right"');
end
N = size(data,2);

if strcmp(direction,'right')
    
    dataLinear = NaN*zeros(N,Ra*dimData);
    for mm = 1:size(data,2)        
        for vv = 1:Ra
            dataLinear(mm,((vv-1)*dimData+1):(dimData*vv)) = (squeeze(data(:,mm,:))*M(:,vv))';
        end
    end
else
    for mm = 1:size(data,2)        
        for vv = 1:Ra
            dataLinear(mm,((vv-1)*dimData+1):(dimData*vv)) = M(:,vv)'*squeeze(data(:,mm,:));
        end
    end
end
zeroCols = sum(abs(dataLinear),1) == 0;
dataLinear(:,zeroCols) = [];
nonZeroCols = find(~zeroCols);
dimLinearData = dimData*Ra;

%dataLinear = dataLinear - repmat(mean(dataLinear,1),N,1);
%dataLinear = dataLinear - repmat(mean(dataLinear,2),1,size(dataLinear,2));


