function [U,V] = estimate_projections(data_train,labels_train,proj_type,modelRank)
% Estimate coefficient vectors for Three-way DA and Spectral LDA classifiers.
% These coefficient vectors are used to extract features from training,
% validation and test data prior to classification. 
% 
% Inputs:
%
% data_train - training data set. Note: since the feature extraction 
% procedure is supervised, the coefficient vectors must be estimated 
% based on the training data set.
%
% labels_train - class labels of the training epochs.
% 
% proj_type - feature extraction type used, must be either 
% "threeway_DA" or "spectral_DA".
%
% modelRank - when "threeway_DA" method is selected, modelRank corresponds
% to the number of coefficient vectors estimated. More specifically, 
% modelRank = R, a parameter matrices "U" and "V" both consists of R
% vectors.
% 
% Outputs:
% 
% U - coefficient vectors "U"
% V - coefficient vectors "V"
%
% Note! When Spectral LDA projection type is chosen, projections are stored
% only in "V". When Three-way DA projection type is chosen, projections are
% stored in both "U" and "V".

U = [];
V = [];

Lab = unique(labels_train);
nrCategories = length(Lab);
if nrCategories == 2; nrCategories = 1;end

switch proj_type
    case 'threeway_DA'
        
        Uf = [];
        Vf = [];
        
        for m = 1:nrCategories
            labelsTmp = zeros(size(labels_train));
            labelsTmp(labels_train == Lab(m)) = 1;
            CL0 = squeeze(mean(data_train(:,labelsTmp == 0,:),2));
            CL1 = squeeze(mean(data_train(:,labelsTmp == 1,:),2));
            meanDif = CL1 - CL0;
            [U,D,V] = svd(meanDif);
            Uf = [Uf U(:,1:modelRank)];
            Vf = [Vf V(:,1:modelRank)];
        end
        
        % reorganize features as:
        % (factor 1, class 1), (factor 1, class2), (factor 2, class1), ...
        UfTmp = [];
        VfTmp = [];
        for m = 1:modelRank
            UfTmp = [UfTmp Uf(:,m:modelRank:size(Uf,2))];
            VfTmp = [VfTmp Vf(:,m:modelRank:size(Uf,2))];
        end
        U = UfTmp;
        V = VfTmp;
        
    case 'spectral_LDA'
        data_train = permute(data_train,[3 2 1]);
        p = size(data_train,1); % nr of frequency bins
        c = size(data_train,3); % nr of channels/ICs
        n = size(data_train,2); % nr of windows
        
        V = zeros(nrCategories,p,c);
        regOpt = 'infinite';
        
        for chanNr = 1:c % estimate coefficients for each channel/IC separately
            D = squeeze(data_train(:,:,chanNr))'; % n*p data matrix
            for m = 1:nrCategories
                labelsTmp = binarize_labels(labels_train,m);
                V(m,:,chanNr) = compute_LDA(D,labelsTmp,regOpt);
            end
        end
    otherwise
        error('Wrong projection type!')
end

