function [data_train,data_validation,data_test,labels_train,labels_validation,labels_test,A,W] = preprocess_data(data_train,data_validation,data_test,labels_train,labels_validation,labels_test,params)
% This function preprocesses data before passing the data for classifiers.
% Preprocessing is based on Short-time Fourier transform and independent component
% analysis (ICA is optional).
% 
% Inputs: 
%
% data_train - training data
% data_validation - validation data
% data_test - test data
% 
% data sets must be provided in one of the three formats:
% 
% 1.Raw format, where an input data set is a C by M real-valued matrix, 
% where C is the number of sensors and M is the length of the time-series 
% (i.e., a total number of time-points). In this case, category labels must
% be provided as an M-dimensional vector, i.e., each time-point must have a
% category label.
% 
% 2. Epoch format, where an input data set is a T by N by C real-valued 
% tensor, where T is the length of the epoch (i.e., time-window), N is the
% is the number of epochs and C is the number of sensors. In this case, 
% category labels must be provided as an N-dimensional vector, i.e., each 
% epoch must have a category label available.
%
% 3. Spectral format, where an input data sets is a F by N by C 
% complex-valued tensor, where F is the number of frequency bins in the 
% epoch, N is the is the number of epochs and C is the number of sensors. 
% In this case, category labels must be provided as an N-dimensional 
% vector, i.e., each epoch must have a category label available. 
% This representation is like Epoch format, but each epoch has been 
% transformed into the Fourier-domain. 
%
% labels_train - class labels for training data
% labels_validation - class labels for validation data
% labels_test - class labels for test data
%
% Class label format: Category labels must be whole numbers from the set:
% {1,2,3,...,K}, where K is the total number of categories.
%
% params is a struct containing preprocessing parameters:
% params.general.do_ica             perform (true) or don't perform (false) ICA
% params.general.samplfreq          sampling frequency in Hz
% params.general.minfreq            minimum frequency in Hz
% params.general.maxfreq            maximum frequency in Hz
% params.general.windowlength_sec   time-window length in seconds
% params.general.overlapfactor      window step size given by windowlength/overlapfactor
% params.ica.pcadim                 PCA dimension (corresponds here to the number of independent components) 
% params.ica.complexmixing          estimate complex- (true) or real-valued (false) mixing matrix
% params.ica.maxiter                maximum number of iterations used in ICA

if length(size(data_train)) == 2
    epochs = 0; % data provided in raw format (channels * time-points)
elseif length(size(data_train)) == 3
    epochs = 1; % data provided in epoch format
else
    error('Wrong dimension of the data!')
end

relevantCoefficients = [];
A = [];

do_ica = params.general.do_ica;

% check if data is already in Spectral format:
if isreal(data_train(:)) 
    do_stft = 1; % run STFT, since data is not in spectral format
else
    do_stft = 0; % do not run STFT, since data is already in spectral format
end

% *************************************************************************
% Short-time Fourier transform:
% *************************************************************************
if do_stft % do short-time Fourier transform
    
    samplfreq = params.general.samplfreq;
    minfreq = params.general.minfreq;
    maxfreq = params.general.maxfreq;
    % When data not given in epoch format, define time-window parameters:
    if ~epochs
        windowlength_sec = params.general.windowlength_sec;
        overlapfactor = params.general.overlapfactor;
    else
        windowlength_sec = [];
        overlapfactor = [];        
    end
    
    [FT_train,TD_train,labels_train] = extract_windows(data_train,labels_train,windowlength_sec,overlapfactor,samplfreq,minfreq,maxfreq);
    [FT_validation,TD_validation,labels_validation] = extract_windows(data_validation,labels_validation,windowlength_sec,overlapfactor,samplfreq,minfreq,maxfreq);
    [FT_test,TD_test,labels_test] = extract_windows(data_test,labels_test,windowlength_sec,overlapfactor,samplfreq,minfreq,maxfreq);

else % no short-time Fourier transform
    FT_train = data_train;
    FT_validation = data_validation;
    FT_test = data_test;
end


if do_ica % run ICA
    
    pcadim = params.ica.pcadim;
    complexmixing = params.ica.complexmixing;
    maxiter = params.ica.maxiter;
    
    % *************************************************************************
    % COMBINE DATA
    % *************************************************************************
    windows_train = size(FT_train,2); % number of training samples
    windows_validation = size(FT_validation,2); % number of validation samples
    
    FT_all = cat(2,FT_train,FT_validation,FT_test);
    
    % **************************************************************
    % Estimate ICA model:
    % **************************************************************
    [IC_all,A,W] = fourierica_custom(FT_all,pcadim,samplfreq,minfreq,maxfreq,...
        'complexmixing',complexmixing,'maxiter',maxiter);
    % **************************************************************
    % Extract training, validation and test data:
    % **************************************************************
    data_train = IC_all(:,1:windows_train,:);
    data_validation = IC_all(:,(windows_train+1):(windows_train+windows_validation),:);
    data_test = IC_all(:,(windows_train+windows_validation+1):end,:);
    
else % no ICA
    data_train = FT_train;
    data_validation = FT_validation;
    data_test = FT_test;
end

