function features = project_data(data,U,V,proj_type)
% This function extracts features from data using linear projections. 
%
% Inputs:
%
% data - a data set to be projected. data is a C*N*F matrix, where:
% C = the number of ICs or channels
% N = the number of time-windows
% F = the number of frequency bins
%
% U,V - coefficient vectors, i.e., the directions of projections.
%
% Note! When Spectral LDA projection type is chosen, projections are stored
% only in "V". When Three-way DA projection type is chosen, projections are
% stored in both "U" and "V".
% 
% proj_type - feature extraction type used, must be either 
% "threeway_DA" or "spectral_DA".
% 
% Outputs:
%
% features - projected epochs. These projections will be used as 
% classifier inputs

switch proj_type
    case 'threeway_DA'
        
        features = zeros(size(U,2),size(data,2))';
        for m = 1:size(data,2)
            Dat = squeeze(data(:,m,:));
            features(m,:) = diag(U'*Dat*V);
        end
    case 'spectral_LDA'
        
        data = permute(data,[3 2 1]);
        
        p = size(data,1);
        n = size(data,2);
        c = size(data,3);
        
        nrFeatures = size(V,1); % number of estimated directions
        
        dataProj = zeros(nrFeatures,c,n);
        iter = 1;
        for m = 1:nrFeatures
            for k = 1:c
                w = squeeze(V(m,:,k));
                if sum(w) == 0
                    w = randn(size(w));
                end
                w = w/norm(w);
                w = w(:);
                X = data(:,:,k)';
                Y = X*w;
                dataProj(m,k,:) = Y;
            end
        end
        
        features = tensor_to_matrix(dataProj);
        
        
    otherwise
        error('Wrong projection type!')
end