function [resu,Classifier_final] = train_and_test_sparse_bilinear(data_train,data_validation,data_test,labels_train,labels_validation,labels_test,params_sparse_bilinear) 
% Train, validate and test sparse bilinear classifier.
%
% Inputs: 
% data_train - training data
% data_validation - validation data
% data_test - test data
%
% Data format: Data is a tensor of size C*N*F, where:
% C is the total number of channels or independent components 
% N is the total number of epochs 
% F is the number of frequency bins in each epoch.
%
% labels_train - class labels for training data
% labels_validation - class labels for validation data
% labels_test - class labels for test data
%
% Class label format: labels are provided as a vector of length N 
% denoting class labels of each epoch. Categories are indexed as 
% 1,2,3,...,K, where K is the total number of categories.
%
% params_sparse_bilinear is a struct containing classifier parameters
%
% Outputs:
%
% resu is a struct containing the test results:
% res.acc           Accuracy for the test data
% res.labelsEstim   Estimated class labels
% res.labelsTrue    True class labels
%
% Classifier_final is a struct containing the final classifier parameters:
% Classifier_final.V                        Vector of parameters V
% Classifier_final.W                        Vector of Parameters W
% Classifier_final.Rank                     Rank of the classifier
% Classifier_final.lambdas1                 Best regularization parameter
% Classifier_final.lambdas2                 Best regularization parameter
% Classifier_final.number_of_categories     a total number of categories

lambdas_comp = params_sparse_bilinear.lambdas_comp;
lambdas_freq = params_sparse_bilinear.lambdas_freq;
modelRank = params_sparse_bilinear.modelRank;

nrPairs = numel(lambdas_comp)*numel(lambdas_freq);
lambda_pairs = zeros(nrPairs,2);
lambdas_comp = sort(lambdas_comp,'descend');
validation_accuracy = zeros(1,nrPairs);
clear Classifier
iter = 1;
% train and validate classifier for each hyperparameter combination:
for m = 1:length(lambdas_comp)
    for n = 1:length(lambdas_freq)
        % train classifier:
        Classifier{iter} = train_sparse_bilinear(data_train,...
            labels_train,lambdas_freq(n),lambdas_comp(m),modelRank);
        % compute accuracy for validation data:
        resu = test_sparse_bilinear(data_validation,labels_validation,Classifier{iter});
        % save validation accuracy and lambda values:
        validation_accuracy(iter) = resu.acc;
        lambda_pairs(iter,:) = [lambdas_freq(n) lambdas_comp(m)];
        iter = iter + 1;
    end
end

% Obtain the best model according to the validation results.
[bestValid,bestValidInd] = max(validation_accuracy);
bestLambdas = lambda_pairs(bestValidInd,:);

% Train final classifier:
Classifier_final = train_sparse_bilinear(cat(2,data_train,data_validation),...
    [labels_train labels_validation],bestLambdas(1),bestLambdas(2),modelRank);

% evaluate final classification accuracy for independent test data set:
resu = test_sparse_bilinear(data_test,labels_test,Classifier_final);
