function Classifier = train_sparse_bilinear(data,labels,lambdas1,lambdas2,modelRank)
% train sparse bilinear classifier 

% Inputs:
% 
% data - training data for the classifier. This is a tensor of size C*N*F, where:
% C is the total number of channels or components
% N is the total number of epochs
% F is the number of frequency bin in each epoch
% 
% labels - a vector of length N denoting class labels of each epoch. 
% Categories are indexed as 1,2,3,...,K, where K is the total number of categories.
%
% lamdas1 - regularization parameters for component coefficients
% lamdas2 - regularization parameters for frequency coefficients
% lamdas1 or lamdas2 can be scalars, vectors, or matrices.
% A row vector of length K means that each category has a same regularization
% sequence. A matrix is of size K*L, where specific regularization sequence 
% is set for each category K.
%
% modelRank - rank of the bilinear classifier (number of coefficient vectors)
%
% Outputs:
%
% Classifier - struct containing parameters of the trained classifier

if exist('L1General2_PSSgb.m') ~= 2
    error('Cannot proceed because L1General-package not found from Matlab path. The package is available in: http://www.cs.ubc.ca/~schmidtm/Software/L1General.zip')
end

% check and set default inputs:
if nargin < 2
    error('Data or category labels not given as an input!')
end

% set default parameters:
switch nargin
    case 2
        lambdas1 = 1; % default regularization
        lambdas2 = 1; % default regularization
        modelRank = 1; % default rank
    case 3
        lambdas1 = 1;
        modelRank = 1;
    case 4
        modelRank = 1;
    case 5    
    otherwise
        error('Wrong number of input parameters!')
end

% Train classifier in a one-versus-rest scheme:
nClasses = max(labels); % total number of categories in the training data
if nClasses == 2 % binary classification is a special case with a single discriminant.
    nClasses = 1;
end

if size(lambdas1,1) > 1 && size(lambdas1,1) ~= nClasses
    error('Lambda sequence must contain hyperparameters in rows, not in columns!')
end
if size(lambdas1,1) > 1 && size(lambdas2,1) ~= nClasses
    error('Lambda sequence must contain hyperparameters in rows, not in columns!')
end
    
if nClasses > 1
    if size(lambdas1,1) == 1
        lambdas1 = repmat(lambdas1,nClasses,1);
    end
    if size(lambdas2,1) == 1
        lambdas2 = repmat(lambdas2,nClasses,1);
    end    
end

data = permute(data,[3 2 1]);
for m = 1:nClasses
    y = 1 + (labels == m); % use binary class indicator vector
    % train classifier:
    [V{m},W{m},pairs] = bilinear_alternate(data,y,modelRank,lambdas1(m,:),lambdas2(m,:));
end

Classifier.V = V;
Classifier.W = W;
Classifier.Rank = modelRank;
Classifier.lambdas1 = lambdas1;
Classifier.lambdas2 = lambdas2;
Classifier.number_of_categories = max(2,nClasses);
