function relevantCoefficients = visualize_patterns_sparse_bilinear(Classifier_final,A,params_sparse_bilinear,pos_map,phasemap_th,max_nr_components,params)
% This function visualizes spectrospatial patterns of the Sparse and
% bilinear classifier.
%
% Inputs: 
%
% Classifier_final - a struct containing trained classifier parameters
% A - spatial pattern estimated by Fourier-ICA
% params_sparse_bilinear - parameters of the classifier, including:
% params_sparse_bilinear.modelRank      rank of the classifier
%
% pos_map - a position map (x,y) of the electrodes
% phasemap_th - threshold for phase maps
% max_nr_components - maximum number of visualized components
% data - data set in a spectral format, i.e., p*n*f, where p is the number
% of components, n is the number of epochs, and f is the number of
% frequency bins.
% labels - a class label vector of length n of the epochs. 
% params - a struct containing general parameters needed, including:
% params.general.minfreq        minimum frequency in Hz
% params.general.maxfreq        maximum frequency in Hz
%
% Outputs:
%
% relevantCoefficients - indexes of components corresponding to nonzero
% classification coefficients (first row) and the value of the
% corresponding classification coefficient (second row).

modelRank = params_sparse_bilinear.modelRank;
nClasses = length(Classifier_final.V);

[V,W] = organize_coefficients(Classifier_final.V,Classifier_final.W,modelRank);
clear relevantCoefficients
for k = 1:nClasses
    coefsW = W{k};
    coefsV = V{k};
    for m = 1:modelRank
        w = coefsW(m,:)';
        F = [find(w) w(find(w))]';
        relevantCoefficients{m}{k} = F;
        v = coefsV(m,:)';
        relevantCoefficientsV{m}(k,:) = v;        
    end
end

% Visualize spatial patterns corresponding to non-zero ICs. One set of
% components is plotted for each category (index k) and each coefficient
% vector (index m).
for m = 1:modelRank
    for k = nClasses:-1:1
        if nClasses == 1
            cate = '1 vs 2';
        else
            cate = num2str(k);
        end        
        if k == 1 && m == modelRank;plot_colorbar = 1;else;plot_colorbar = 0;end        
        if modelRank > 1
            titl = ['Sparse bilinear: Coeff. set ' num2str(m) ', discriminative ICs for category ' cate];
            titl2 = ['Sparse bilinear: Coeff. set ' num2str(m)];
        else
            titl = ['Sparse bilinear: Discriminative ICs for category ' cate];
            titl2 = ['Sparse bilinear'];
        end
        visualize_spatial_patterns(relevantCoefficients{m}{k},phasemap_th,max_nr_components,A,pos_map,plot_colorbar,titl);
    end
     visualize_spectral_patterns(relevantCoefficientsV{m},params,titl2,max_nr_components);
end

function visualize_spectral_patterns(ffilt,params,titl,max_nr_components)

fr = linspace(params.general.minfreq,params.general.maxfreq,size(ffilt,2));
figure
%set(gcf,'Position',[10 450 920 230],'Name','Sparse and bilinear classifier')
set(gcf,'Name','Sparse and bilinear classifier')
for m = 1:size(ffilt,1)
    subplot(size(ffilt,1),1,m)
    plot(fr,ffilt(m,:),'b.-');grid on
    if m == 1
        if size(ffilt,1) == 1
           title(['Spectral filter for category 1 vs 2']) 
        else
           title(['Spectral filters for each category']) 
        end
    end
    if m == size(ffilt,1)
        xlabel('freq (Hz)')
    end
end

function visualize_spatial_patterns(ICvec,phasemap_th,max_nr_components,A,pos_map,plot_colorbar,titl)
% Visualization of the spatial patterns (magnitude and phase) of the 
% independent components.

% Inputs:
%
% ICvec - A two-row matrix where the first row contains indices of the independent
% components (ICs), and the second row contains the corresponding classification
% coefficient of the ICs.
% A - spatial patterns of all the ICs
% pos_map - position map of the electrode array
% plot_colorbar - plot (plot_colorbar = 1) or don't plot (plot_colorbar = 0) colorbar.
% titl - name of the figure

IC_index = ICvec(1,:);
coeffVals = abs(ICvec(2,:));

% plot at most 10 components:
[coeffVals,sinds] = sort(coeffVals,'descend');
IC_index = IC_index(sinds);
if length(IC_index) > max_nr_components
    coeffVals = coeffVals(1:max_nr_components);
    IC_index = IC_index(1:max_nr_components);
%    warning(['Plotting ' num2str(max_nr_components) ' most relevant components only, increase "max_nr_components" to allow for more plots...'])
end
largeInds = find(coeffVals>0);

coeffVals = coeffVals(largeInds);
IC_index = IC_index(largeInds);

% plot magnitude maps:
figure
set(gcf,'Position',[30 70 610 560],'name',titl)
for m = 1:length(coeffVals)
    if length(IC_index) > 4 && length(IC_index) < 9 
        subplot(4,2,m)        
    else
        subplot(length(IC_index),1,m)
    end
    ICApatterns = A(:,IC_index(m));    
    % actual plotting commands are executed here:
    plot_array(ICApatterns','magnitude',pos_map);
    title(['IC #' num2str(IC_index(m)) ', coeff: ' num2str(round(100*coeffVals(m))/100)])
end
cmap1 = get(gcf,'Colormap');

% plot phase maps:
figure
set(gcf,'Position',[670 70 610 560],'name',titl)
for m = 1:length(coeffVals)
    if length(IC_index) > 4 && length(IC_index) < 9 
        subplot(4,2,m)
    else
        subplot(length(IC_index),1,m)
    end
    ICApatterns = A(:,IC_index(m));    
    plot_array(ICApatterns','phase',pos_map,phasemap_th);
    title(['IC #' num2str(IC_index(m)) ', coeff: ' num2str(round(100*coeffVals(m))/100)])
end
cmap2 = get(gcf,'Colormap');

% plot colorbars:
if plot_colorbar
    figure;set(gcf,'Colormap',cmap1,'Name','Magnitude colorbar of Sparse bilinear');
    h = colorbar('NorthOutside');set(h,'XTickLabel',[])
    title('Colorbar for magnitude plots: [0,...,1]');axis off;    
    xt=[0:0.1:1];
    set(h,'XTick',linspace(1,65,11),'XTickLabel',xt)
    figure;set(gcf,'Colormap',cmap2,'Name','Phase colorbar of Sparse bilinear');
    h = colorbar('NorthOutside');set(h,'XTickLabel',[])
    title('Colorbar for phase plots: [-\pi/2,...,\pi/2]');axis off;
    set(gcf,'Position',[400 30 560 420])
    xt=[{'-pi/2'},{'-3pi/8'},{'-pi/4'},{'-pi/8'},{'0'},{'pi/8'},{'pi/4'},{'3pi/8'},{'pi/2'}];
    set(h,'XTick',linspace(1,65,9),'XTickLabel',xt)
end





