function relevantCoefficients = visualize_patterns_spectral_LDA(Classifier_final,A,pos_map,phasemap_th,max_nr_components,data,labels,params)

% This function visualizes spectrospatial patterns of the Spectral LDA
% classifier.
%
% Inputs: 
%
% Classifier_final - a struct containing trained classifier parameters
% A - spatial pattern estimated by Fourier-ICA
% pos_map - a position map (x,y) of the electrodes
% phasemap_th - threshold for phase maps
% max_nr_components - maximum number of visualized components
% data - data set in a spectral format, i.e., p*n*f, where p is the number
% of components, n is the number of epochs, and f is the number of
% frequency bins.
% labels - a class label vector of length n of the epochs. 
% params - a struct containing general parameters needed, including:
% params.general.minfreq        minimum frequency in Hz
% params.general.maxfreq        maximum frequency in Hz

if iscell(Classifier_final.beta)
    nClasses = length(Classifier_final.beta);
else
    nClasses = 1;
end

clear relevantCoefficients relevantData
for k = 1:nClasses
    if nClasses == 1
        best_betas = Classifier_final.beta(:,Classifier_final.bestClassifierIndex);
    else
        best_betas = Classifier_final.beta{k}(:,Classifier_final.bestClassifierIndex);        
    end
    best_betas_organized = reshape(best_betas,nClasses,length(best_betas)/nClasses);
    w = best_betas_organized(k,:)';
    F = [find(w) w(find(w))]';
    nr_comp = size(F,2);
    if nr_comp == 0
        continue
    end
    if size(F,2) == 1
        S = squeeze(Classifier_final.spectralProjections(k,:,F(1,:)))';
    else
        S = squeeze(Classifier_final.spectralProjections(k,:,F(1,:)));        
    end
    F = [F ; S];
    relevantCoefficients{k} = F;
    CL1 = squeeze(mean(data(F(1,:),labels==k,:),2));
    CL0 = squeeze(mean(data(F(1,:),labels~=k,:),2));
    if nr_comp == 1
        CL0n = normalize_vectors(CL0,'columns');
        CL1n = normalize_vectors(CL1,'columns');
    else
        CL0n = normalize_vectors(CL0,'rows');
        CL1n = normalize_vectors(CL1,'rows');        
    end
    %CL0s = standardize_vectors(CL0,'rows');
    %CL1s = standardize_vectors(CL1,'rows');
    relevantData{k,1} = CL1n;
    relevantData{k,2} = CL0n;
end

% Visualize spatial patterns corresponding to non-zero ICs. One set of
% components is plotted for each category (index k) and each coefficient
% vector (index m).
for k = nClasses:-1:1
    if ~isempty(relevantData{k,1})
        if k == 1 ;plot_colorbar = 1;else;plot_colorbar = 0;end
        if nClasses == 1
            titl = ['Spectral LDA: Discriminative components for category 1 vs 2'];
        else
            titl = ['Spectral LDA: Discriminative components for category ' num2str(k)];        
        end
        visualize_spatial_and_spectral_patterns(relevantCoefficients{k},relevantData{k,1},relevantData{k,2},phasemap_th,A,pos_map,plot_colorbar,titl,params,max_nr_components);
    else
        if nClasses == 1
            warning(['No class-specific features found for category 1 vs 2 in Spectral LDA classifier'])
        else
            warning(['No class-specific features found for category ' num2str(k) ' in Spectral LDA classifier'])            
        end
    end
end

function visualize_spatial_and_spectral_patterns(ICvec,dataMean1,dataMean2,phasemap_th,A,pos_map,plot_colorbar,titl,params,max_nr_components)

% Visualization of the spatial patterns (magnitude and phase) of the
% independent components.

% Inputs:
%
% ICvec - A two-row matrix where the first row contains indices of the independent
% components (ICs), and the second row contains the corresponding classification
% coefficient of the ICs.
% A - spatial patterns of all the ICs
% pos_map - position map of the electrode array
% plot_colorbar - plot (plot_colorbar = 1) or don't plot (plot_colorbar = 0) colorbar.
% titl - name of the figure

IC_index = ICvec(1,:);
coeffVals = abs(ICvec(2,:));
freqCoeff = ICvec(3:end,:)';
% ignore components with very small coefficient values:
% coeffVals = coeffVals(coeffVals>0.05);

% sort ICs according to coefficients from highest to lowest:
if length(coeffVals) > 1
    [coeffVals,sinds] = sort(coeffVals,'descend');
    IC_index = IC_index(sinds);
    freqCoeff = freqCoeff(sinds,:);
    dataMean1 = dataMean1(sinds,:);
    dataMean2 = dataMean2(sinds,:);
    
    if length(IC_index) > max_nr_components
        coeffVals = coeffVals(1:max_nr_components);
        IC_index = IC_index(1:max_nr_components);
        warning(['Plotting ' num2str(max_nr_components) ' most relevant components only, increase "max_nr_components" to allow for more plots...'])
    end
    largeInds = find(coeffVals>0);    
    coeffVals = coeffVals(largeInds);
    IC_index = IC_index(largeInds);
    freqCoeff = freqCoeff(largeInds,:);
    dataMean1 = dataMean1(largeInds,:);
    dataMean2 = dataMean2(largeInds,:);    
else
   dataMean1 = dataMean1';
   dataMean2 = dataMean2';    
end

% plot magnitude maps:
figure
set(gcf,'Position',[30 70 610 560],'name',titl)
ind = 1;
fr = linspace(params.general.minfreq,params.general.maxfreq,size(dataMean1,2));
for m = 1:length(coeffVals)
    subplot(length(IC_index),3,ind)
    ICApatterns = A(:,IC_index(m));
    % actual plotting commands are executed here:
    plot_array(ICApatterns','magnitude',pos_map);
    title(['IC #' num2str(IC_index(m)) ', coef ' num2str(round(100*coeffVals(m))/100)])
    subplot(length(IC_index),3,ind+1)
    plot(fr,freqCoeff(m,:),'b.-');grid on;
    if m == 1
        title('Spectral filter of IC')
    end
    if m == length(IC_index)
        xlabel('freq (Hz)')
    end
    subplot(length(IC_index),3,ind+2)
    %    title('Spectra of component: Class of interest = red Others = black')
    plot(fr,dataMean1(m,:),'r.-');hold on;plot(fr,dataMean2(m,:),'k.-');grid on;
    if m == 1
        title([{'Spectrum of IC'};{'red=class of interest'};{'black=others'}])
    end
    if m == length(IC_index)
        xlabel('freq (Hz)')
    end    
    ind = ind + 3;
    % alternatively, plot component index with the stability value:
    % title(['Component #' num2str(coefAll(m)) ', magnitude, stability: ' num2str(compSorted(m)/maxCount)])
    
end
cmap1 = get(gcf,'Colormap');

% plot phase maps:
if sum(abs(imag(ICApatterns(:)))) ~= 0
    figure
    set(gcf,'Position',[670 70 610 560],'name',titl)
    for m = 1:length(coeffVals)
        subplot(length(IC_index),1,m)
        ICApatterns = A(:,IC_index(m));
        plot_array(ICApatterns','phase',pos_map,phasemap_th);
        title(['Phase of component #' num2str(IC_index(m)) ', coeff: ' num2str(round(100*coeffVals(m))/100)])
    end
    cmap2 = get(gcf,'Colormap');
end
% plot colorbars:
if plot_colorbar
    figure;set(gcf,'Colormap',cmap1,'Name','Magnitude colorbar of Spectral LDA');
    h = colorbar('NorthOutside');set(h,'XTickLabel',[])
    title('Colorbar for magnitude plots: [0,...,1]');axis off;
    xt=[0:0.1:1];
    set(h,'XTick',linspace(1,65,11),'XTickLabel',xt)    
    if sum(abs(imag(ICApatterns(:)))) ~= 0
        figure;set(gcf,'Colormap',cmap2);
        h = colorbar('NorthOutside');set(h,'XTickLabel',[])
        title('Colorbar for phase plots: [-\pi/2,...,\pi/2]');axis off;
        set(gcf,'Position',[400 30 560 420],'Name','Phase colorbar of Spectral LDA')
        % xt=[{'-pi/2'},{'-pi/4'},{'0'},{'pi/4'},{'pi/2'}];
        % set(h,'XTick',linspace(1,65,5),'XTickLabel',xt)
        xt=[{'-pi/2'},{'-3pi/8'},{'-pi/4'},{'-pi/8'},{'0'},{'pi/8'},{'pi/4'},{'3pi/8'},{'pi/2'}];
        set(h,'XTick',linspace(1,65,9),'XTickLabel',xt)
    end
end
